#ifndef __BUZZER_H
#define __BUZZER_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "Debug.h"
#include "tim.h"
//  <o>蜂鸣类型配置
//      <1=>1：单音（固定4kHz）
//      <2=>2：美音（外置金属音）
//      <3=>3：电源板发音
#define SOUND_SIGLE 1   // 单音（固定4kHz）
#define SOUND_CHORD 2   // 美音（外置金属音）
#define SOUND_POWER 3   // 电源板发音

#define SOUND_TYPE 2


#define EOT  0xFFFF /*(8MHz/32/((ff+1)*2) = ?à 488.28125Hz) */
#define SOL_LO 1506// 82  /*1.506kHz) */
#define RA_LO 1666// 74  /*1.666kHz) */
#define SI_LO 1866// 66  /*1.866kHz)*/
#define DO_LO 1984// 62     /*1.984KHz)*/
#define RE  2273// 54  /*2.273KHz)*/
#define MI  2551// 48  /*2.551KHz)*/
#define PA  2717// 45  /*2.717KHz)*/
#define SOL  2976// 41  /*2.976KHz)*/
#define RA  3289// 37  /*3.289KHz)*/
#define SI  3676// 33  /*3.676KHz)*/
#define DO_HI 3906// 31  /*3.906KHz)*/
#define RE_HI 4464// 27  /*4.464KHz)*/
#define MI_HI 5000// 24  /*5.000KHz)*/

/** @brief 蜂鸣状态 */
typedef enum 
{
	BUZZ_DIS,		///<静音
	BUZZ_ON,		///<播放
	BUZZ_OFF,		///<暂停
}BuzzStates_E;

/** @brief 蜂鸣类型 */
typedef enum
{
	BUZZ_SINGLE,	///<单音
	BUZZ_MUSIC,		///<音乐
}BuzzType_E;
/** @brief 曲谱参数结构 */
typedef struct
{
    UINT16       u8Pitch;    ///<音符
    UINT8       u8OnTime;   ///<播放时间
    UINT8       u8OffTime;  ///<停顿时间    
}MusicPara_T;     

/**
*	@brief	蜂鸣器IO开启
*	@author	toby
*	@retval	无
*	@note
*			宏函数作用：\n
*			1、使能音源输出；\n
*			2、电源控制口打开。
*/
	#define	BuzzOn()	do{						\
							HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_SET);	\
						}while(0)
/**
*	@brief	蜂鸣器IO关闭
*	@author	toby
*	@retval	无
*	@note
*			宏函数作用：\n
*			1、使能音源输出；\n
*			2、电源控制口关闭。
*/
	#define	BuzzOff()	do{						\
							HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_RESET);	\
						}while(0)  

#define  BUZZ_FRQ_SET(FRQ)   do{            \
	__HAL_TIM_SET_AUTORELOAD(&htim15, (1000000 / FRQ));     \
}while(0)

extern MusicPara_T TAB_CUSTOM_MUSIC[];

void	Buzzer_Init(void);
UINT8   BuzzMusicSet(MusicPara_T *_musicData);
void    Buzzer_Process();

#ifdef __cplusplus
 }
#endif
#endif

