/*
*************************************************************************************************
*
*	Copyright (C), 2022-2050, www.hdsmart.net.cn
*
*	module:				Debug
*	author:				wangyu
*	version:				1.0
*	description:		Provide debugging and printing functions
*	release history:
*								Date 						Name				description
*      							2024-06-04			wangyu			V1.0:Initial version
*
*
**************************************************************************************************
*/
#include "Debug.h"
#include "usart.h"
#include <stdarg.h>


static const char *gs_colourString[] = {
    "\033[34m",
    "\033[30m",
    "\033[31m",
    "\033[33m",
    "\033[32m",
    "\033[30m",
};

static const char gs_logString[] = {
    "MCEWID",
};
int Debug_Send(int len,char * buf)
{
	if(len > 0)
		HAL_UART_Transmit(&hlpuart1, (UINT8 *)buf, len, 100);

    return len;
}

/*
***************************************************************************************
*
*	@brief  		Debugging function initialization
* 	@param		None
* 	@retval		None
*
***************************************************************************************
*/
void Debug_Init(void)
{

	HAL_UART_Transmit(&hlpuart1, (UINT8 *)"\033[2J\033[1H", sizeof("\033[2J\033[1H"), 100);

}

/*
***********************************************************************************************************
*
*	@brief  		Debugging print functions
* 	@param		_type:	RTOS type
* 						_level:	_level
* 						_format:	Format parameters
* 	@retval		Number of bytes sent
*
***********************************************************************************************************
*/
static UINT8 gs_debugBuf[DEBUG_BUFFER_SIZE];	/*  */

UINT32 Debug_Printf(UINT8 _type,UINT8 _level,const char * _format, ... )
{
	UINT32 bufSize,cnt = 0;
    va_list   v_args;
	
	bufSize = DEBUG_BUFFER_SIZE;

    cnt = snprintf((char       *)&gs_debugBuf[0], bufSize,(char const *) "%s",gs_colourString[_level]);
	
	if(_level != LOG_COMMON_LEVEL)
	{
		cnt += snprintf((char       *)&gs_debugBuf[cnt], bufSize,(char const *) "[%c]   ",gs_logString[_level]);
	}
 
    va_start(v_args, _format);
    cnt += vsnprintf((char      *)&gs_debugBuf[cnt],(bufSize - cnt), \
                    (char const *) _format,  v_args);
    va_end(v_args);

	cnt += snprintf((char       *)&gs_debugBuf[cnt], bufSize,(char const *) "\033[30m");
	
	{

		Debug_Send(cnt,(char *)gs_debugBuf);

	}	
	return cnt;
}



